; Filenoter
;
; - Explorer context menu action to create and open a note file
; - example: the default notefile for aaa.pdf is aaa.pdf.txt
; - options: custom tags, datestamps, extensions (see ini file for details)
;
; version 180222
; by Nod5

#SingleInstance ignore


default_ini =
(
; filenoter configuration file
;
; - By default the filnote for a file named "aaa.pdf" is "aaa.pdf.txt"
; - To change any option first remove the semicolon ; at the line start
; then follow the instructions below
;
; OPTION -- INSTRUCTION
;
; tag    -- Add a suffix tag to note names
;        -- example: with "tag=--NOTE" the filenote for "aaa.pdf" is "aaa.pdf--NOTE.txt"
;
; date   -- Add a datestamp suffix tag to note names
;        -- two possible formats:
;        -- date=1 means datestamp format YYMMDD-HHMM
;        -- date=2 means datestamp format YYMMDD
;        -- example: with "date=2" the filenote for "aaa.pdf" is "aaa.pdf180204.txt"
;
; update -- if old filenote with datestamp suffix already exist
;        -- then update datestamp in its filename. Don't create a new (second) filenote
;        -- example: "update=1"
;
; ext    -- Use custom extension name for notes, instead of .txt
;        -- example: with "ext=notes" the filenote for "aaa.pdf" is "aaa.pdf.notes"
;
; infolder  -- place filenotes for folders inside the folder
;           -- example: with "infolder=1" the filenote for
;                      folder "C:\dir" is "C:\dir\dir.txt"
;
; subfolder -- place filenotes for files in a subfolder
;           -- example: with "subfolder=info" the filenote for
;                       "C:\aaa.pdf" is "C:\info\aaa.pdf.txt"


[options]
;tag=--note
;date=
;update=
;ext=
;infolder=
;subfolder=
)


param = %1%    ;expect path to right clicked file
filepath := param

if !param                  ;no param
  goto register_filenoter  ;add/remove Explorer context menu action registry keys

;else create a filenote

If !FileExist(filepath)
  exitapp


If FileExist(A_ScriptDir "\filenoter.ini")
{
  IniRead, xtag, %A_ScriptDir%\filenoter.ini, options, tag, %A_Space%
  IniRead, xdate, %A_ScriptDir%\filenoter.ini, options, date, %A_Space%
  IniRead, xupdate, %A_ScriptDir%\filenoter.ini, options, update, %A_Space%
  IniRead, xext, %A_ScriptDir%\filenoter.ini, options, ext, %A_Space%
  IniRead, xinfolder, %A_ScriptDir%\filenoter.ini, options, infolder, %A_Space%
  IniRead, xsubfolder, %A_ScriptDir%\filenoter.ini, options, subfolder, %A_Space%

  ;trim forbidden filename characters
  xtag := RegExReplace(xtag, "(?:\\|/|:|\*|\?|""|<|>)","")
  xext := RegExReplace(xext, "(?:\\|/|:|\*|\?|""|<|>)","")
}
else
{
  ;create default ini and show it to user
  FileAppend, %default_ini%, %A_ScriptDir%\filenoter.ini
}



;tag option
if !xtag
  xtag := ""  ;default


;ext option
if !xext
  xext := "txt"  ;default


;infolder option
if (xinfolder == 1) and InStr( FileExist(filepath), "D") ;folder
{
  SplitPath, filepath, xdirname, xpathdir
  filepath := filepath "\" xdirname  
  ;example: filenote for folder C:\abc is C:\abc\abc.txt
}


;subfolder option
if xsubfolder and !InStr( FileExist(filepath), "D") ;not for folders
{
  SplitPath, filepath, xname, xdir
  filepath := xdir "\" xsubfolder "\" xname
  If !FileExist(xdir "\" xsubfolder)
    FileCreateDir, % xdir "\" xsubfolder
  ;example: C:\aaa.pdf --> C:\info\aaa.pdf.txt  if xsubfolder=info
}


;date option
if !(xdate == 1 or xdate == 2)
  xdate := ""
if (xdate == 1)
  xnow := SubStr(A_YYYY,3) A_MM A_DD "-" A_Hour A_Min   ;YYMMDD-HHMM
if (xdate == 2)
  xnow := SubStr(A_YYYY,3) A_MM A_DD                    ;YYMMDD


;assemble new filenote path from all above options
newfilenote := filepath xtag xnow "." xext

;remove any double backslash
newfilenote := StrReplace(newfilenote, "\\","\")

;update option
;if date option and previous filenote already exists
;then update its datestamp rather than create a new file
if xupdate and xdate
  Loop, files, %filepath%%xtag%*.%xext% , FD  ;no recurse
  {
    trimlen := StrLen( xnow "." xext )
    newfilestub  := SubStr(newfilenote       ,1,-trimlen)   ;trim away <datestamp>.<ext>
    loopfilestub := SubStr(A_LoopFileFullPath,1,-trimlen)
    ;example: C:\dir\aaa.pdf--NOTE171224.txt --> C:\dir\aaa.pdf--NOTE

    if (newfilestub != loopfilestub)
      continue

    do_update := ""
    if (xdate == 1)
      if RegExMatch(A_LoopFileFullPath, "^.+\d\d\d\d\d\d-\d\d\d\d." xext "$")
        do_update := 1
    if (xdate == 2)
      if RegExMatch(A_LoopFileFullPath, "^.+\d\d\d\d\d\d" xext "$")
        do_update := 1

    if !do_update
      continue

    ;update filenote datestamp and open
    FileMove, % A_LoopFileFullPath , % newfilenote , 1
    Run, Notepad "%newfilenote%"
    exitapp
  }


;else if filenote exist open it
if FileExist(newfilenote)
{
Run, Notepad "%newfilenote%"
exitapp
}

;else make new filenote and open
FileAppend,, % newfilenote
Run, Notepad "%newfilenote%"
ExitApp


;add/remove Explorer context menu action registry keys
register_filenoter:

If !FileExist(A_ScriptDir "\filenoter.ini")
  FileAppend, %default_ini%, %A_ScriptDir%\filenoter.ini

if !A_IsAdmin
  {
  ;need to be administrator to edit registry in Windows 10
  msgbox, to add/remove Filenoter in context menu, run it as Administrator
  exitapp
  }
  
RegRead, xreg, HKEY_LOCAL_MACHINE\SOFTWARE\Classes\*\shell\FileNoter
if (ErrorLevel == 1)  ;not found, so create it
{
  msgbox, 1,Filenoter, Create FileNoter context menu action for Explorer?
  ifmsgbox cancel
    ExitApp
  RegWrite, REG_SZ, HKEY_LOCAL_MACHINE\SOFTWARE\Classes\*\shell\FileNoter,,FileNoter
  RegWrite, REG_SZ, HKEY_LOCAL_MACHINE\SOFTWARE\Classes\*\shell\FileNoter\command,,%A_ScriptFullPath% "`%L"
  RegWrite, REG_SZ, HKEY_LOCAL_MACHINE\SOFTWARE\Classes\Directory\shell\FileNoter,,FileNoter
  RegWrite, REG_SZ, HKEY_LOCAL_MACHINE\SOFTWARE\Classes\Directory\shell\FileNoter\command,,%A_ScriptFullPath% "`%L"
  
  ;if no ini, create it and show to user
  if !FileExist(A_ScriptDir "\filenoter.ini")
    FileAppend, % default_ini , % A_ScriptDir "\filenoter.ini"
  
  msgbox,
  (
  FileNoter context menu entry created
  Using path %A_ScriptFullPath% 
  
  - To remove run Filenoter again as Administrator
  - To change options edit filenoter.ini
  )
  
}
else if (xreg == "FileNoter")
{
  msgbox, 1,Filenoter, Remove FileNoter context menu action for Explorer?
  ifmsgbox cancel
    ExitApp
  RegDelete, HKEY_LOCAL_MACHINE\SOFTWARE\Classes\*\shell\FileNoter
  RegDelete, HKEY_LOCAL_MACHINE\SOFTWARE\Classes\Directory\shell\FileNoter
  msgbox, FileNoter context menu entry removed.

}

exitapp
